// Sieve of Eratosthenes to generate the prime numbers from 2..N
// This program generates the prime numbers between 2 and N, then counts them and prints the count

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <iostream>

// accepts N, the upper bound to count primes

int main (int argc, char **argv) {
	if (argc != 2) {
		std::cerr << "Usage: " << argv[0] << " <n>\n";
		return 1;
	}

	// get N

	int n;
	sscanf (argv[1], "%d", &n);

	// get an array of N booleans

	bool *A = new bool[n];
	if (!A) {
		std::cerr << "allocation failed!\n";
		return 1;
	}

	// mark all the booleans as 1, i.e. each number is initially presumed to be prime

	memset (A, 1, n);

	// for all numbers up to the square root of N, mark off array indices that are multiples of that number

	int lim = sqrt (n);
	for (int skip=2; skip<=lim; skip++) {
		for (int i=skip*2; i<n; i+=skip) A[i] = false;
	}

	// count up all the numbers left unmarked; they are not multiples of anything so they must be prime

	int count = 0;
	for (int i=2; i<n; i++) count += A[i];
	printf ("there are %d prime numbers less than %d\n", count, n);
	return 0;
}
